//This code captures a set number of video frames and corresponding time stamps from a Basler USB3 camera. 
//The actual camera parameters (frame rate, image size, gain etc.) are saved in the camera firmware using a predefined 'userset'.
//This userset can be created or modified using the Basler Pylon software (https://www.baslerweb.com/en/products/software/basler-pylon-camera-software-suite/) or NI Measurement and Automation Explorer (NI MAX)

//Required libraries:
//	Basler Pylon API (https://www.baslerweb.com/en/products/software/basler-pylon-camera-software-suite/)
//	Serialization

//Please email Benjamin Smith (bsmith@rvc.ac.uk) to report bugs or with any questions
//Code copyright Benjamin Smith 2019
//This work is licensed under a Creative Commons Attribution 4.0 International License.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Basler.Pylon;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;


namespace getHSvideo
{
    static class Program
    {
        [STAThread]

        public static void Serialize(object t, string path)
        {
            using (Stream stream = File.Open(path, FileMode.Create))
            {
                BinaryFormatter bformatter = new BinaryFormatter();
                bformatter.Serialize(stream, t);
            }
        }

        
       

        static void Main()
        {
            long vtime = DateTime.Now.Ticks; //time stamp
            int c = 600; //number of images to save - divide by frame rate to get length of video in seconds
            string path = "C:/Users/admin/Documents/Gondola 5/HS camera/getHSvideo/getHSvideo/bin/Debug/video" + vtime.ToString(); //location to save the video+metadata
            string csvpath = path + "/metadata.csv";
            bool exists = System.IO.Directory.Exists(path);

            if (!exists)
                System.IO.Directory.CreateDirectory(path);



            using (Camera camera = new Camera("21784519"))//open the camera with the serial number "21784519" (or whatever the serial number of your camera is) - use NI Max or Basler Pylon to find the serial number of your camera
            {
                camera.CameraOpened += Configuration.AcquireContinuous;

                camera.Open();

                camera.Parameters[PLCamera.UserSetSelector].SetValue(PLCamera.UserSetSelector.UserSet1); //load a userset - this defines the dimensions of each video frame and the frame rate, and can be edited using Basler Pylon or NI Max
                camera.Parameters[PLCamera.UserSetLoad].Execute();
                camera.Parameters[PLCamera.ChunkModeActive].SetValue(true);

                camera.Parameters[PLCamera.ChunkSelector].SetValue(PLCamera.ChunkSelector.Timestamp);
                camera.Parameters[PLCamera.ChunkEnable].SetValue(true);
                camera.Parameters[PLCameraInstance.MaxNumBuffer].SetValue(1000); //needs to be greater than the number of frames you will be saving to maintain consistent frame rate

                Console.WriteLine("USERSET 1 LOADED");
                Console.WriteLine("Using camera {0}.", camera.CameraInfo[CameraInfoKey.ModelName]);

                var csv = new StringBuilder();
                var imgpx = new StringBuilder();

                camera.StreamGrabber.Start();
              
                byte[][] vid_buffer = new byte[c][];   // Grab a number of images into the buffer

                for (int i = 0; i < c; i++)
                {
                    
                    IGrabResult grabResult = camera.StreamGrabber.RetrieveResult(5000, TimeoutHandling.ThrowException); // Wait for an image and then retrieve it. A timeout of 5000 ms is used.

                    using (grabResult)
                    {
             
                        if (grabResult.GrabSucceeded)
                        {
                            
                            if (grabResult.ChunkData[PLChunkData.ChunkTimestamp].IsReadable)//get the timestamp of each video frame and add to the metadata file (required for synchronisation with wheel speed and force)
                            {
                                var first = grabResult.ChunkData[PLChunkData.ChunkTimestamp];
                                var newLine = string.Format("{0}{1}", first, Environment.NewLine);
                                csv.Append(newLine);
                            }
                            else
                            {
                                Console.WriteLine("ChunkData: No TimeStamp");
                            }

                            vid_buffer[i] = grabResult.PixelData as byte[];//get the actual camera data

                        }
                        else
                        {
                            Console.WriteLine("Error: {0} {1}", grabResult.ErrorCode, grabResult.ErrorDescription);
                        }
                    }

                }

                camera.StreamGrabber.Stop();
                camera.Close();

                System.IO.File.WriteAllText(csvpath, csv.ToString());//metadata is saved to HDD
                Serialize(vid_buffer, path + "/imgpxdata.dat"); //video data is saved to HDD


                Console.WriteLine("Data saved");

            }
            System.Windows.Forms.Application.Exit();

        }
    }
}
