%This code compares the time stamps of the video metadata files and the wheel tdms files,
%and produces a list of which videos correspond to which tdms files.
%In order to reduce the time taken, only files between the start and end dates will be compared.
%The function also filters out which videos are unlikely to produce good data by only listing those
%where the file is complete, where a mouse is in frame for most of the video, 
%and where the mean speed of running is greater than 0.1m/s.

%N.B. This code was tested using white (FVB) mice, the 5% threshold for selecting videos may not apply to mice of different colours!

%Function inputs:
%	tdms_fl - path of folder with tdms files (string)
%	video_fl - path of folder with video folders (string)
%	startdate - beginning of time period to compare (inclusive of this date) (string formatted as dd-mm-yy)
%	enddate - end of time period to compare (inclusive of this date) (string formatted as dd-mm-yy)

%Function outputs:
%	vidsync - tdms file names and corresponding video names (cell)

%Required libraries:
%	TDMS reader by Jim Hokanson (https://www.mathworks.com/matlabcentral/fileexchange/30023-tdms-reader)
%	peakdet by Marco Borges (https://www.mathworks.com/matlabcentral/fileexchange/47264-peakdet)

%Please email Benjamin Smith (bsmith@rvc.ac.uk) to report bugs or with any questions
%Code copyright Benjamin Smith 2019
%This work is licensed under a Creative Commons Attribution 4.0 International License.

function vidsync = lv_vid_sync(tdms_fl,video_fl,startdate,enddate)


%-------------------get LV start and end times---------------------------

l_lv = dir(tdms_fl);%get list of folders in that directory
nf = length(l_lv) %number of files 
 
sd_lv = 3;

%get start and end indices
for g = 3:length(l_lv)%index starts from 3 as the first three fieldnames are 'name', 'Props' and 'Time'
    fname = l_lv(g).name;
    fdate = datenum(fname(11:18),'yy-mm-dd'); 
    if (fdate>=datenum(startdate,'dd-mm-yy'))&&(fdate<datenum(enddate,'dd-mm-yy'))
        sd_lv = g;
        break
    end
end

for g = 3:length(l_lv)%index starts from 3 as the first three fieldnames are 'name', 'Props' and 'Time'
    fname = l_lv(g).name;
    fdate = datenum(fname(11:18),'yy-mm-dd');
    if (fdate>datenum(enddate,'dd-mm-yy'))
        ed_lv = g-2;
        break
    end
    ed_lv = length(l_lv);
end

%get LV times and mean speeds to check the data is worth looking at
for j = sd_lv:ed_lv
    j2 = j-2; %this is just so I don't have to keep writing out j-2
    fd = l_lv(j).name

    if (char(fd(end)) == 's')&&(l_lv(j).bytes>44000)%make sure there a useful amount of data in the file
        fpath = strcat(tdms_fl,'\',fd);
        
        try
        d = TDMS_getStruct(fpath,4); %read the file
        dn = fieldnames(d);
		
        d_first = strcat('d.',char(dn(2)));
        d_last = strcat('d.',char(dn(end)));
        ttemp_first = eval(strcat(d_first,'.Time.data'));
        ttemp_last = eval(strcat(d_last,'.Time.data'));
        tvec_first = datevec(ttemp_first);
        hoff = tvec_first(1,4)-str2num(fd(20:21));
		
        lvstart(j) = addtodate(ttemp_first(1),-hoff,'hour');
        lvend(j) = addtodate(ttemp_last(end),-hoff,'hour');
    
        for n = 3:length(dn)
            dd = strcat('d.',char(dn(n)));
            dpath = strcat(dd,'.Voltage_0.data');
            if (n == 3)
                z = eval(dpath);
            else
                z = horzcat(z,eval(dpath));
            end
        end
        
        pks = peakdet(z,0.015);
        
        if length(pks)>2
            locs = pks(:,1);
            mv_rps(j) = 0.04*2*pi*mean((1/4)./(diff(locs)/3000));%calculate mean speed for file (in m/s - 0.04 is the radius of the wheel)
        else
            mv_rps(j)  =NaN;
        end
        
        catch
        warning(strcat('Bad file at ',char(fd)));
        end
        
            
    else
        lvstart(j) = NaN;
        lvend(j) = NaN;
        mv_rps(j)  =NaN;
    end

end

% %--------------------Get video start and end times--------------------------------------
l_v = dir(video_fl);%get list of folders in that directory
nf = length(l_v) %number of folders

%get start and end indices
datenum(startdate,'dd-mm-yy');
datenum(enddate,'dd-mm-yy');

sd_v = 3;

for g = 3:length(l_v)
    fname = l_v(g).name;
    str2num(fname(6:end));
    fdate = addtodate(str2num(fname(6:end))/(24*60*60*1e7),1,'year');%convert C# time to MATLAB time
	%fdate = addtodate(fdate,2,'day') %convert C# time to MATLAB time
	if(fdate>=datenum(startdate,'dd-mm-yy'))&&(fdate<datenum(enddate,'dd-mm-yy'))
        sd_v = g;
        break
    end
end

for g = 3:length(l_v)
    fname = l_v(g).name;
    fdate = addtodate(str2num(fname(6:end))/(24*60*60*1e7),1,'year');%convert C# time to MATLAB time
	%fdate = addtodate(fdate,2,'day') %convert C# time to MATLAB time
    if (fdate>datenum(enddate,'dd-mm-yy'))
        ed_v = g-1;
        break
    end
    ed_v = length(l_v);
end

for j = sd_v:ed_v
    j2 = j-2; 
    fd = l_v(j).name;%get start time stamp from folder name - this is created just *before* grabbing frames, so will be at the start
    endtick = System.IO.File.GetLastWriteTime(strcat(video_fl,'\video',fd(6:end),'\metadata.csv'));
    vidend(j) = addtodate(cast(endtick.Ticks,'double')/(24*60*60*1e7),367,'day');
    vidstart(j) = addtodate(vidend(j),-3,'second');
end

%----------------Check that video file is complete and mouse is in frame------------------------------

for i = sd_v:ed_v
    length(dir(strcat(video_fl,'\',char(l_v(i).name)))) 
    if (length(dir(strcat(video_fl,'\',char(l_v(i).name)))) > 3)%check that both .dat and metadata files exist
        imgpx = dir(strcat(video_fl,'\',char(l_v(i).name),'\imgpxdata.dat'));
        imgsize = imgpx.bytes;%calculate the size of the .dat file (smaller than 400kb indicates it did not complete writing and is unlikely to convert to avi correctly)
        fid = fopen(strcat(video_fl,'\',char(l_v(i).name),'\imgpxdata.dat'),'r');
        allnum = fread(fid,Inf,'*uint8');
        C = hist(cast(allnum,'double'),50);
        C = C ./ sum(C);%calculate the relative proportions of each grey level in the video (less than 5% white [1,1,1] indicates the mouse was not in frame for a large proportion of the video)
        fclose all;
    else
        imgsize = 0;
    end
    for j = sd_lv:ed_lv%match the videos and TDMS files which pass the desired filters
        if ((vidstart(i)>=lvstart(j))&&(vidstart(i)<lvend(j)))&&(imgsize >= 400000000)&&(C(end)>0.05)&&(mv_rps(i)>=0.1)
            vidsync{i,1} = char(l_v(i).name);
            vidsync{i,2} = char(l_lv(j).name);
        elseif ((vidend((i))>=lvstart(j))&&(vidend(i)<lvend(j)))&&(imgsize >= 400000000)&&(C(end)>0.05)&&(mv_rps(i)>=0.1)
            vidsync{i,1} = char(l_v(i).name);
            vidsync{i,2} = char(l_lv(j).name);
        end
    end
end

vidsync %print out values


            





    