%This code combines the output files from the digitised videos, the video metadata, and the variables calculated using 'wheelLVdataproc.m'
%to produce arrays of stance time, swing time, stride times, duty factor and peak vertical ground reaction force, and the corresponding speeds.

%For each video-force dataset you should have:
%	a .csv file that contains the frame numbers of each foot on and foot off time (the program assumes the first element is a foot on, and the last element is foot off, so you end up with one more stance value than swing or stride)
%	a .csv file with the video metadata
%	the corresponding TDMS file
%Place all the foot contact data in a folder called footcontacts, all the metadata in a folder called metadata, and all the TDMS files in a folder called LVdata 

%Function inputs:
%	folder_frames - path of folder with metadata 
%	folder_footcontacts - path of folder with foot contacts
%	et - continuous timestamp vector for each TDMS file (from 'wheelLVdataproc.m')
%	v_rps - continuous speed vector for each TDMS file (from 'wheelLVdataproc.m')
%	grf - continuous force vector for each TDMS file (from 'wheelLVdataproc.m')

%Function outputs:
%	speed - vector of mean speed across a stance period in m/s (double)
%	stance - vector of stance times in seconds (double)
%	swing - vector of swing times in seconds (double)
%	stride - vector of stride times in seconds (double)
%	DF - vector of duty factors (stance time/ stride time) (double)
%	pkGRF - vector of peak force across a stance period in m/s (double)

%Please email Benjamin Smith (bsmith@rvc.ac.uk) to report bugs or with any questions
%Code copyright Benjamin Smith 2019
%This work is licensed under a Creative Commons Attribution 4.0 International License.

function [speed,stance,swing,stride,DF,pkGRF] = speedstancesync(folder_frames, folder_footcontacts, et, v_rpsin,grfin)

lf = dir(folder_frames);%get list of folders in that directory
lfc = dir(folder_footcontacts);%get list of folders in that directory
llv = dir(strcat(folder_frames(1:end-8),'LVdata'));
nf = length(lf) %number of files 
nfc = length(lfc) %number of files 

for i = 3:nf

    grf = grfin(i-2,:);
    vrps = (v_rpsin(i-2,:));

    ftns1 = 0;
    ftns2 = 0;
    st = 0;
    sw = 0;
    str = 0;
    spd = 0;
    pkf = 0;
    
%-----------Align foot on and off times with equivalent times in speed and GRF vectors--------------------
    fpath = strcat(folder_frames,'\',lf(i).name);
    fcpath = strcat(folder_footcontacts,'\',lfc(i).name);
    lvname = llv(i).name;
    lvdatetime = datenum(lvname(11:21),'yy-mm-dd_HH');
    lvdatetimedata = datevec(et{i-2}(1));


     if (lvdatetimedata(5)>=30)
         toff(i-2) = 1+etime(datevec(lvdatetime),lvdatetimedata)/(60*60);
         yes = 1;
     else
        toff(i-2) = etime(datevec(lvdatetime),lvdatetimedata)/(60*60);
        no = 1;
     end
    
    frames = csvread(fpath);
    footcontacts =  csvread(fcpath);
    
    fname = lfc(i).name;
    wticks = str2num(fname(6:23));
    mlticks = (wticks+(24*60*60*1e7*367))/(24*60*60*1e7);%convert windows ticks to matlab ticks;
    mlticks = addtodate(mlticks,-1,'second');
    
	for j = 1:length(footcontacts)

		if footcontacts(j)>0%find equivalent foot on and off times 
			ftns1(j) = addtodate(mlticks,(footcontacts(j))*5,'millisecond');
			ftns2(j) = ((wticks+(j-1)*5e4)+(24*60*60*1e7*367))/(24*60*60*1e7);
		else
			ftns1(j) = NaN;
			ftns2(j) = NaN;
		end
	end


	for b = 1:length(et{i-2})
		LVT(b) = addtodate(et{i-2}(b),round(toff(i-2)),'hour');
	end

	vstart = datestr(ftns1(1),'dd-mm-yy HH:MM:SS.FFF');%get video start time
	vend = datestr(ftns1(end),'dd-mm-yy HH:MM:SS.FFF');%get video end time
	lstart = datestr(LVT(1),'dd-mm-yy HH:MM:SS.FFF');%get TDMS start time
	lend = datestr(LVT(end),'dd-mm-yy HH:MM:SS.FFF');%get TDMS end time  
   
	for k = 1:2:length(ftns1)-1%sort foot on times into column 1 and foot off times into column 2
		st(k,1) = ftns1(k);
		st(k,2) = ftns1(k+1);
	end
    
	st = st(any(st,2),:);%remove all zero rows
	
%------------Calculate variable values-------------------------------------------------

	for h = 1:length(st)
		[sc,si] = min(abs(st(h,1)-LVT));%get start of each stance period
		[fc,fi] = min(abs(st(h,2)-LVT));%get start of each swing period
		spd(h) = mean(vrps(si:fi));%get mean speed for each period
		
		if isnan(mean(grf(si:fi)))%get peak GRF for each period
			pkf(h) = NaN;
		else
			pkf(h) = max(grf(si:fi));
		end
		
	end


	speed{i-2} = spd;
	pkGRF{i-2} = pkf;
	stance{i-2} = etime(datevec(st(:,2)),datevec(st(:,1)));
	
	for m = 2:length(st)
		sw(m-1) = etime(datevec(st(m,1)),datevec(st(m-1,2)));
		str(m-1) = etime(datevec(st(m,1)),datevec(st(m-1,1)));
	end
	sw(m) = NaN;%add extra element to swing and stance vectors to bring length in line with stance vector
	str(m) = NaN;

	swing{i-2} = sw;
	stride{i-2} = str;

	DF{i-2} = stance{i-2}./stride{i-2}';

end





    
  