%This code reads a folder of TDMS data files recorded by the running wheel and calculates speed, vertical ground reaction force
%and running times.

%Function inputs:
%	fl_data - path of folder where TDMS files are saved (string)
%	f_cal - path of the calibration .txt file (string)

%Function outputs:
%	v_rps - continuous speed vector for each TDMS file in m/s (double)
%	mv_rps - mean speed for each TDMS file in m/s (double)
% 	mv - mean hourly speed in m/s (double)
%	bins - number of times each hour the wheel was triggered (double)
%	circ - total number of seconds spent running per hour (double)
%	grf - continuous vertical ground reaction force vector for each TDMS file in gram-force (double)
%	et - continuous timestamp vector for each TDMS file (double)

%Required libraries:
%	TDMS reader by Jim Hokanson (https://www.mathworks.com/matlabcentral/fileexchange/30023-tdms-reader)
%	peakdet by Marco Borges (https://www.mathworks.com/matlabcentral/fileexchange/47264-peakdet)

%Please email Benjamin Smith (bsmith@rvc.ac.uk) to report bugs or with any questions
%Code copyright Benjamin Smith 2019
%This work is licensed under a Creative Commons Attribution 4.0 International License.

function [v_rps, mv_rps, mv, bins, circ, grf, et] = wheelLVdataproc(fl_data,f_cal)

grf = 0;


k = [0,1,2,3,4,5,6,7,8,9]; %channel numbers


l = dir(fl_data);%get list of folders in that directory
nf = length(l) %number of files

bins = zeros(24,1); %24 hours
circ = zeros(24,1); %24 hours - length of time spent running per hour
hs = zeros(24,1);
si = ones(24,1);
bl = ones(1,nf); %bout lengths
bc = 1; %bout counter

fileID = fopen(f_cal,'r'); %read in calibration values
formatSpec = '%f %f';
fsize = [2 Inf];
chs = fscanf(fileID,formatSpec,fsize)';
fclose(fileID);

%-------------Read in all data---------------------------------------------

for j = 3:nf
    j2 = j-2 %this is just so I don't have to keep writing out j-2
    fd = l(j).name
    date_time(j2,:) = fd(11:23);
    fpath = strcat(fl_data,'\',fd);
    
    try
        d = TDMS_getStruct(fpath,4); %read the file
        
        dn = fieldnames(d);
        
        dd = strcat('d.',char(dn(2)));
        dd2 = fieldnames(eval(dd));
        c = length(fieldnames(eval(dd)))-3; %number of channels - first three fieldnames are 'name', 'Props' and 'Time'
        
        for i = 1:c %get data from individual channels
            fnt2 = char(dd2(i+3));
            dpath = strcat(dd,'.',fnt2,'.data');
            temp = eval(dpath);
            tzi(i,1:100) = temp(1:100);
        end
        
        ttemp = eval(strcat(dd,'.Time.data')); %get timestamp
        et{j2} = ttemp;
        z{j2} = tzi;
        
        for n = 3:length(dn)
            dd = strcat('d.',char(dn(n)));
            dd2 = fieldnames(eval(dd));
            c = length(fieldnames(eval(dd)))-3;
            %number of channels - first three fieldnames are 'name', 'Props' and 'Time'
            
            for i = 1:c %get data from individual channels
                fnt2 = char(dd2(i+3));
                dpath = strcat(dd,'.',fnt2,'.data');
                temp = eval(dpath);
                tz(i,1:100) = temp(1:100);
            end
            
            ttemp = eval(strcat(dd,'.Time.data')); %get timestamp
            et{j2} = horzcat(et{j2},ttemp);
            z{j2} = horzcat(z{j2},tz);
        end
        
    catch
        warning(strcat('Bad file at ',char(fd)));
    end
    
end

for j = 3:nf
    
    j2 = j-2 %this is just so I don't have to keep writing out j-2
    
    gt = (z{j2});
    
    if (size(gt)~=[0,0])
        
        v1 = gt(1,:);
        
        %-----------------------Calculate times and bouts-----------------------------------
        h = str2num(date_time(j2,9:10))+1 % hour value - 1 is added because the first element is going to be midnight i.e. hour 00, and 0 cannot be used as an index
        m = str2num(date_time(j2,11:12))+1; % current minute
        
        bins(h) = bins(h)+1; %number of triggers per hour
        circ(h) = circ(h)+(nnz(v1)/3000);%time in seconds run per hour
        
        
        %---------------Calculate vertical ground reaction forces-------------------------------------------
        
        f_run = ((chs(1,1)*v1+chs(1,2)));
        
        for i = [2,3,4,5,6,7,8,9,10]
            v = gt(i,:);
            f_run = vertcat(f_run,((chs(i,1)*v+chs(i,2))));
        end
        
        f = max(f_run,[],1);
        
        fp = peakdet(smooth(f),0.01);
        if (length(fp)>2)
            grf_temp = spline(fp(:,1),fp(:,2),[1:length(f_run)]); %complete force trace
            grf(j2,[1:length(f_run)]) = grf_temp;
        else
            grf(j2,[1:length(v1)]) = zeros(1,length(v1));
        end
        
        %------------Calculate speeds---------------------------------------------------
        
        pv = peakdet(v1,0.01);
        r = 0.04; %running radius in m - this is typical for our mice, but you might want to adjust it if your mice act differently
        
        if (length(pv)>2) %if there are only 2 peaks there is not enough data to estimate speed
            locs = pv(:,1);
            v_rps(j2,[1:length(v1)]) = 0.04*2*pi*interp1(locs(2:end),((1/16)./(diff(locs)/3000)),[1:length(v1)],'pchip'); %connect all the speed points smoothly
            vv = v_rps(j2,[1:length(v1)]); %this is just for checking the existence of data
        else
            vv = [];
            v_rps(j2,[1:length(v1)]) = zeros(1,length(v1));
        end
        
        if(~isempty(vv))
            mv_rps(j2) = 2*pi*r*mean((1/16)./(diff(locs)/3000));
            
            if(mv_rps(j2)>0)
                hs(h) = hs(h)+mv_rps(j2);
                si(h) = si(h)+1;
            end
        end
    end
end

mv = hs./si; %mean speed per hour

figure %plot mean speed
plot(mv_rps*0.04*2*pi)
title('Mean speed for each 5s block(m/s)');

figure %plot mean speed per hour
plot([0:23],mv)
title('Mean speed for each hour (m/s)');
xlabel('Time (BST)');
ylabel('Speed (m/s)');

figure %plot preferred speeds
[N, X] = hist(mv_rps(mv_rps>0)*0.04*2*pi);
bar(X, 100*(N./sum(N)))
title('Preferred speed (m/s)');
xlabel('Speed (m/s)');
ylabel('Frequency (% of running time)');

figure %plot number of events
bar([0:23],circ./7)
title('Average time spent running each hour');
xlabel('Time (BST)');
ylabel('Time (s)');

end


